#!/usr/bin/env python
"""
U2FsdGVkX19cV2plTMSqbpdFaJuEChbUka1NX+X4qeSMMFuTquH1Wd+vL6e2Hf4H
CAvb+R3+MDOCjdOyhgS3DNleCDqZRRJorq7lehTQzbA/pY9TMQBMyUnm63wUOjd/
EKVAggOM+jODpiFn0+FN4le8LUoOErVSZSR7L4gka/7L69wT82r90gq12H1mFNns
judHXORdh3Iud6J8UQHqSZUSFRfnd1IetIIySIIVxxCPE0DY2CS7Y/TjmEiHduzm
l4TiHrn9MlQOJY21PIjswrB9EIS/fsQsNV09BIBh6cJTu2jPULRjOpVGw+liyvkJ
I6HynL0O7XEVC61l2wiQJA==
"""

"""
Single dispatch of available PSBP messages, keyed by command.

"""

import warnings

from construct.core import FormatFieldError
from construct.core import IndexFieldError
from construct.core import StopFieldError

from . import current_msg as cur
from . import frequency_msg as fre
from . import i_max_msg as i_max
from . import mode_msg as mod
from . import phase_msg as pha
from . import pll_offset_msg as pll
from . import power_msg as pwr
from . import status_msg as sta
from . import temp_msg as tem
from . import version_msg as ver
from . import voltage_msg as vol

_PSBP_TABLE = dict(sta.msg_classes.items()
                   + ver.msg_classes.items()
                   + pwr.msg_classes.items()
                   + mod.msg_classes.items()
                   + tem.msg_classes.items()
                   + vol.msg_classes.items()
                   + fre.msg_classes.items()
                   + cur.msg_classes.items()
                   + i_max.msg_classes.items()
                   + pha.msg_classes.items()
                   + pll.msg_classes.items())


class InvalidPSBPMessageType(NotImplementedError):
    """
    Base exception for messages with invalid message types.
    """


def dispatch(msg, table=_PSBP_TABLE):
    """
    Dispatch an PSBP message type based on its `command` and parse its
    payload.

    Parameters
    ----------
    msg : :class:`PSBP`
      A parsed PSBP object.
    table : dict
      Any table mapping unique PSBP message type IDs to PSBP message
      constructors.

    Returns
    ----------
    PSBP message with a parsed payload.

    """
    try:
        return table[msg.command](msg)
    except KeyError:
        warn = "No message found for command id %d for msg %s." \
               % (msg.command, msg)
        warnings.warn(warn, RuntimeWarning)
        return msg
    except FormatFieldError:
        warnings.warn("PSBP payload deserialization error! 0x%x" % msg.command,
                      RuntimeWarning)
        return msg
    except IndexFieldError:
        warnings.warn("PSBP payload deserialization error! 0x%x" % msg.command,
                      RuntimeWarning)
        return msg
    except StopFieldError:
        warnings.warn("PSBP payload deserialization error! 0x%x" % msg.command,
                      RuntimeWarning)
        return msg
